using DAL;
using DAL.Models;
using Infrastructure.EFCore.Query;
using Microsoft.EntityFrameworkCore;

namespace Infrastructure.EFCore.Test
{
    public class QueryTests
    {
        private readonly SchoolDBContext dbContext;

        public QueryTests()
        {
            var myDatabaseName = "mydatabase_" + DateTime.Now.ToFileTimeUtc();

            var dbContextOptions = new DbContextOptionsBuilder<SchoolDBContext>()
                            .UseInMemoryDatabase(databaseName: myDatabaseName)
                            .Options;

            dbContext = new SchoolDBContext(dbContextOptions);

            dbContext.Classrooms.Add(new Classroom { Id = 1, Name = "B113" });
            dbContext.Classrooms.Add(new Classroom { Id = 2, Name = "B117" });
            dbContext.Classrooms.Add(new Classroom { Id = 3, Name = "A217" });

            dbContext.Students.Add(new Student { Id = 1, Name = "Jozko Mrkvicka" });
            dbContext.Students.Add(new Student { Id = 2, Name = "Ferko Hruska" });
            dbContext.Students.Add(new Student { Id = 3, Name = "Misko Jablko" });
            dbContext.Students.Add(new Student { Id = 4, Name = "Jozko Chladnicka" });
            dbContext.Students.Add(new Student { Id = 5, Name = "Ferko Azerbajan" });
            dbContext.Students.Add(new Student { Id = 6, Name = "General Kenobi" });
            dbContext.Students.Add(new Student { Id = 7, Name = "Han Solo" });
            dbContext.Students.Add(new Student { Id = 8, Name = "Han Crew" });
            dbContext.Students.Add(new Student { Id = 9, Name = "Han Singleplayer" });

            dbContext.SaveChanges();
        }

        [Fact]
        public void ClassroomA217Exists_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<string>(a => a == "B113", "Name");
            var result = efquery.Execute();

            Assert.True(result.Count() == 1);

            Assert.True(result.First().Name == "B113");
        }

        [Fact]
        public void ClassroomsThatStartWithBExists_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<string>(a => a.StartsWith("B"), "Name");
            var result = efquery.Execute();

            Assert.True(result.Count() == 2);
        }

        [Fact]
        public void ClassroomsWithIDLessThan2_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<int>(a => a < 3, "Id");
            var result = efquery.Execute();

            Assert.True(result.Count() == 2);
        }

        [Fact]
        public void ClassroomsOrderedAscending_QueryOrderBy_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.OrderBy<int>("Id", true);
            var result = efquery.Execute()
                .Select(a => a.Id)
                .ToList();

            var ExpectedResult = dbContext.Classrooms
                .Select(a => a.Id)
                .OrderBy(a => a)
                .ToList();

            Assert.Equal(ExpectedResult, result);
        }

        [Fact]
        public void ClassroomsOrderedDescending_QueryOrderBy_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.OrderBy<int>("Id", false);
            var result = efquery.Execute()
                .Select(a => a.Id)
                .ToList();

            var ExpectedResult = dbContext.Classrooms
                .Select(a => a.Id)
                .OrderByDescending(a => a)
                .ToList();

            Assert.Equal(ExpectedResult, result);
        }

        [Fact]
        public void StudentsSimplePagination_QueryPagination_Test()
        {
            var efquery = new EntityFrameworkQuery<Student>(dbContext);
            efquery.Page(3, 3);
            var result = efquery.Execute()
                .Select(a => a.Id)
                .ToList();

            var ExpectedResult = dbContext.Students
                .Skip(6)
                .Take(3)
                .Select(a => a.Id)
                .ToList();

            Assert.Equal(ExpectedResult, result);
        }

        [Fact]
        public void QueryAdvancedTest1()
        {
            var efquery = new EntityFrameworkQuery<Student>(dbContext);
            efquery.Where<int>(a => a < 7, "Id");
            efquery.OrderBy<string>("Name", true);
            efquery.Page(1, 4);
            var result = efquery.Execute().ToList();

            var ExpectedResult = dbContext.Students
                .Where(a => a.Id < 7)
                .OrderBy(a => a.Name)
                .Take(4)
                .ToList();

            Assert.Equal(ExpectedResult, result);
        }
    }
}